function FearConditioning
% SETUP
% BNC1 connected to Arduino UNO for video recording trigger
% BNC2 connected to Shocker for shocker trigger

global BpodSystem

%% Define parameters
S = BpodSystem.ProtocolSettings; % Load settings chosen in launch manager into current workspace as a struct called S
if isempty(fieldnames(S))  % If settings file was an empty struct, populate struct with default settings
    
    S.SoundDuration = 30;
    S.ShockDuration=2;
    S.ITI=[10, 2, 5,3, 6, 2,5,2,4,8, 2, 5,3, 6, 2,5,2,4,8];
%     S.soundID=[1,2,1,2,2,1,1,2,2,1];
    S.soundID=[2,2,2,2,2,2,2,2,2,2,2,2,2,2,2];
end



BpodSystem.Data.TrialTypes = []; % The trial type of each trial completed will be added here.
BpodSystem.Data.TrialRewarded = []; % The trial type of each trial completed will be added here.
BpodSystem.Data.ITI = [];
%% Initialize plots

% Set soft code handler to trigger sounds
BpodSystem.SoftCodeHandlerFunction = 'SoftCodeHandler_PlaySoundX';

SF = 192000; % Sound card sampling rate
SinWaveFreq1 = 12000;
sounddata1 = GenerateSineWave(SF, SinWaveFreq1, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)
SinWaveFreq2 = 4000;
sounddata2 = GenerateSineWave(SF, SinWaveFreq2, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)

% Program sound server
PsychToolboxSoundServer('init')
PsychToolboxSoundServer('Load', 1, sounddata1);
PsychToolboxSoundServer('Load', 2, sounddata2);

pause(2);
%% Main trial loop
for currentTrial = 1:10
        % Sync parameters with BpodParameterGUI plugin
      
        soundID=S.soundID(currentTrial);
        switch soundID
            case 1
        ShockerOutput='Wire2';
        TrialTypes = 1;
         case 2
        ShockerOutput='BNC2';
          TrialTypes = 0;
        end
        
        
    
        % Assemble state matrix
        sma = NewStateMatrix(); 
        
        % GlobalTimer1 is used to control the timing of response window.
        sma=SetGlobalTimer(sma,'TimerID', 1, 'Duration',90, 'OnsetDelay',0.001,'Channel','BNC1');
        
        % GlobalTimer1 is used to control the timing of response window.
        sma = SetGlobalTimer(sma, 'TimerID', 2, 'Duration', S.ShockDuration, 'OnsetDelay',28, 'Channel',ShockerOutput);
        
         sma = AddState(sma, 'Name', 'State0', ...
            'Timer', 2,...
            'StateChangeConditions', {'Tup', 'TrialStart'},...
            'OutputActions', {'BNC1',0});
        
        %State#1
        sma = AddState(sma, 'Name', 'TrialStart', ...
            'Timer', 30,...
            'StateChangeConditions', {'Tup', 'CS'},...
            'OutputActions', {'GlobalTimerTrig',1});
        
        sma = AddState(sma, 'Name', 'CS', ...
            'Timer', S.SoundDuration,...
            'StateChangeConditions', {'Tup', 'PostCS'},...
            'OutputActions', {'SoftCode', soundID,'GlobalTimerTrig',2});   
        
                sma = AddState(sma, 'Name', 'PostCS', ...
            'Timer', 0,...
            'StateChangeConditions', {'GlobalTimer1_End', 'ITI'},...
            'OutputActions', {}); 
        
        
        
        sma = AddState(sma, 'Name', 'ITI', ...
            'Timer', S.ITI(currentTrial),...
            'StateChangeConditions', {'Tup', 'exit'},...
            'OutputActions', {});
    
    
    SendStateMatrix(sma);
    RawEvents = RunStateMatrix;
    
    if ~isempty(fieldnames(RawEvents)) % If trial data was returned
        BpodSystem.Data = AddTrialEvents(BpodSystem.Data,RawEvents); % Computes trial events from raw data
        BpodSystem.Data.TrialSettings(currentTrial) = S; % Adds the settings used for the current trial to the Data struct (to be saved after the trial ends)
        BpodSystem.Data.ITI(currentTrial) = S.ITI(currentTrial);
        BpodSystem.Data.TrialTypes(currentTrial) = TrialTypes; % Adds the trial type of the current trial to da
        
        SaveBpodSessionData; % Saves the field BpodSystem.Data to the current data file
    end
    HandlePauseCondition; % Checks to see if the protocol is paused. If so, waits until user resumes.
    if BpodSystem.Status.BeingUsed == 0
        return
    end
end


