function GoNoGoSucroseQuinine

% SETUP

% > Connect the Valve A in the box to Bpod Port#1.
% > Connect lick to Bpod Port#3
% > Connect the Valve B in the box to Bpod Port#4.
% > Connect vacuum valve in the box to Bpod Port#5.
% Modified by Tao for imaging purpose 8/29/2018

global BpodSystem

%% Define parameters
S = BpodSystem.ProtocolSettings; % Load settings chosen in launch manager into current workspace as a struct called S
if isempty(fieldnames(S))  % If settings file was an empty struct, populate struct with default settings
    
    S.GUI.RewardAmount =10; % ul
    S.GUI.PunishAmount = 1; % ul 
    S.GUI.TrialGoProb = 0.5; 
    S.GUI.PreGoTrialNum = 6;
    S.GUI.VacuumOnTime = 0.5;
    S.GUI.VacuumDelay = 2;
    S.GUI.ResponseTimeGo = 1; % How long until the mouse must make a choice, or forefeit the trial
    S.GUI.ResponseTimeNoGo = 1; % How long until the mouse must make a choice, or forefeit the trial
    
    S.GUI.ImagingStart = 1;   S.GUI.ImagingNumber =1;
    
    S.GUI.TrainingLevel = 1; % Configurable training level
    S.GUIMeta.TrainingLevel.Style = 'popupmenu'; % the GUIMeta field is used by the ParameterGUI plugin to customize UI objects.
    S.GUIMeta.TrainingLevel.String = {'1','2','3'};
    
    S.GUI.PunishDelayMean = .5;
    S.GUI.RewardDelayMean = .5;
    S.GUI.VacuumDelayMean = .1;
    S.PunishDelayMax = 1.2;
    S.RewardDelayMax = 1.2;
    S.PunishDelayMin = 0.8;
    S.RewardDelayMin = 0.8;
    
    S.CueDelay = 1.0; % the time from cue to response
    S.ITI = 10;
    S.ITI_min=5; S.ITI_max=15;
    S.SoundDuration = 1.0;
    
    S.RndFlag = 1;
    
end

% Initialize parameter GUI plugin
BpodParameterGUI('init', S);
TotalRewardDisplay('init');

%% Define trials
MaxTrials = 200;

ImagingFlag = zeros(1,MaxTrials);
% ImagingFlag(1,1:4:end)=1;
if S.GUI.TrainingLevel==1
    TrialTypes = ones(1,MaxTrials);
elseif S.GUI.TrainingLevel==2
    TrialTypes = ones(1,MaxTrials)
    ImagingFlag = zeros(1,MaxTrials);
    ImagingFlag(S.GUI.ImagingStart:S.GUI.ImagingNumber -S.GUI.ImagingStart+1) = 1;

elseif S.GUI.TrainingLevel==3

    TrialTypes = ones(1,MaxTrials);
    ImagingFlag(1,1:3:end)=1;   
    for ii=(S.GUI.PreGoTrialNum+1):MaxTrials
        if rand<S.GUI.TrialGoProb
            TrialTypes(ii) = 1;
        else
            TrialTypes(ii) = 0;
        end
    end
end

R = repmat(S.GUI.PunishDelayMean,1,MaxTrials);
if S.PunishDelayMax>S.PunishDelayMin
    for k=1:MaxTrials
        candidate_delay = exprnd(S.GUI.PunishDelayMean);
        while candidate_delay>S.PunishDelayMax || candidate_delay<S.PunishDelayMin
            candidate_delay = exprnd(S.GUI.PunishDelayMean);
        end
        R(k) = candidate_delay;
    end
end
PunishDelay = R;

R = repmat(S.GUI.RewardDelayMean,1,MaxTrials);
if S.RewardDelayMax>S.RewardDelayMin
    for k=1:MaxTrials
        candidate_delay = exprnd(S.GUI.RewardDelayMean);
        while candidate_delay>S.RewardDelayMax || candidate_delay<S.RewardDelayMin
            candidate_delay = exprnd(S.GUI.RewardDelayMean);
        end
        R(k) = candidate_delay;
    end
end
RewardDelay = R;

R = repmat(S.ITI,1,MaxTrials);
for k=1:MaxTrials
    candidate_delay = exprnd(S.ITI);
    while candidate_delay>S.ITI_max || candidate_delay<S.ITI_min
        candidate_delay = exprnd(S.ITI);
    end
    R(k) = candidate_delay;
end

ITI = randi(3,1,MaxTrials);
BpodSystem.Data.TrialTypes = []; % The trial type of each trial completed will be added here.
BpodSystem.Data.TrialRewarded = []; % The trial type of each trial completed will be added here.
BpodSystem.Data.RewardDelay = [];
BpodSystem.Data.PunishDelay = [];
BpodSystem.Data.ITI = [];
%% Initialize plots
BpodSystem.ProtocolFigures.SideOutcomePlotFig = figure('Position', [200 200 1000 300],'name','Outcome plot','numbertitle','off', 'MenuBar', 'none', 'Resize', 'off');
BpodSystem.GUIHandles.SideOutcomePlot = axes('Position', [.075 .3 .89 .6]);
GoNoGoOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'init',TrialTypes);

% Set soft code handler to trigger sounds
BpodSystem.SoftCodeHandlerFunction = 'SoftCodeHandler_PlaySoundX';

SF = 192000; % Sound card sampling rate
SinWaveFreq1 = 10000;
sounddata1 = GenerateSineWave(0.03, SF, SinWaveFreq1, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)
SinWaveFreq2 = 3000;
sounddata2 = GenerateSineWave(0.03, SF, SinWaveFreq2, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)

% Program sound server
PsychToolboxSoundServer('init')
PsychToolboxSoundServer('Load', 1, sounddata1);
PsychToolboxSoundServer('Load', 2, sounddata2);

%% Main trial loop
for currentTrial = 1:MaxTrials
    
    S = BpodParameterGUI('sync', S); % Sync parameters with BpodParameterGUI plugin
    RewardValveTime = GetValveTimes(S.GUI.RewardAmount, 2);  %water:1  sucrose:2
    PunishValveTime = GetValveTimes(S.GUI.PunishAmount, 3);  %% for quinine
    
    
    switch TrialTypes(currentTrial) % Determine trial-specific state matrix fields
        case 1 % go trial
            soundID = 1;
            LickOutcome = 'Reward';
            ValveState = 2;
            ResponseTime = S.GUI.ResponseTimeGo;
        case 0 % no-go trial
            soundID = 2;
            LickOutcome = 'Punishment';
            ValveState = 4;   %% 8 airpuff
            ResponseTime = S.GUI.ResponseTimeNoGo;
    end
    
    sma = NewStateMatrix(); % Assemble state matrix
    
    switch S.GUI.TrainingLevel
    case 1      %% Licking spout to get water %%
        
        sma = AddState(sma, 'Name', 'TrialStart',...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {});
        sma = AddState(sma, 'Name', 'DelayW1', ...
            'Timer', 0,... % time before trial start
            'StateChangeConditions', {'Tup', 'DelayW2'},...
            'OutputActions', {});
       sma = AddState(sma, 'Name', 'DelayW2', ...
            'Timer', 0,... % time before trial start
            'StateChangeConditions', {'Tup', 'DelayW3'},...
            'OutputActions', {});
       sma = AddState(sma, 'Name', 'DelayW3', ...
            'Timer', 0,... % time before trial start
            'StateChangeConditions', {'Tup', 'StimulusDeliver'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'StimulusDeliver', ...
            'Timer', 0,... % Start tone
            'StateChangeConditions', {'Tup', 'CueDelay'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'CueDelay', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'ResponseW1'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'ResponseW1', ...
             'Timer',10,...
            'StateChangeConditions', {'Port1In', 'ResponseW2','Tup', 'ITI' },...
             'OutputActions', {});  
         sma = AddState(sma, 'Name', 'ResponseW2', ...
             'Timer',0,...
            'StateChangeConditions', { 'Tup', 'Delay'},...
             'OutputActions', {});  
        sma = AddState(sma, 'Name', 'Delay', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'Reward'},...
            'OutputActions', {});    
        sma = AddState(sma, 'Name', 'Reward', ...
            'Timer', 0,... % reward delay
            'StateChangeConditions', {'Tup', 'DeliverReward'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'DeliverReward', ...
            'Timer', RewardValveTime,... % reward amount
            'StateChange Conditions', {'Tup', 'VacuumDelay'},...
            'OutputActions', {'ValveState', 1}); % 'SoftCode', soundID
        sma = AddState(sma, 'Name', 'VacuumDelay', ...
            'Timer', S.GUI.VacuumDelay,...
            'StateChangeConditions', {'Tup', 'VacuumOn'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'VacuumOn', ...
            'Timer', S.GUI.VacuumOnTime,... % Time for vacuum to be on and remove residual fluids
            'StateChangeConditions', {'Tup', 'ITI'},...
            'OutputActions', {'ValveState', 16});   %'ValveState', 16
        sma = AddState(sma, 'Name', 'Punishment', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup','ITI'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'ITI', ...
             'Timer', 0.5,...
             'StateChangeConditions', {'Tup', 'exit'},...
             'OutputActions', {});
        
        case 2    %% Training mice to lick spout to get sucrose---Go trial %%       


     if ImagingFlag(currentTrial)==1
            sma = SetGlobalTimer(sma,  'TimerID', 1, 'Duration',20, 'OnsetDelay', 2, 'Channel', 'Wire1');   %18
             sma = SetGlobalTimer(sma,  'TimerID', 2, 'Duration',3,'OnsetDelay', 2); 
            sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {'GlobalTimerTrig', 1});
     else
            sma = SetGlobalTimer(sma,  'TimerID', 2, 'Duration',3,'OnsetDelay', 2); 
            sma = AddState(sma, 'Name', 'TrialStart',...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {});
        end;
        sma = AddState(sma, 'Name', 'DelayW1', ...
            'Timer', 10,... % time before trial start      8
            'StateChangeConditions', {'Tup', 'DelayW2'},...
            'OutputActions', {});
       sma = AddState(sma, 'Name', 'DelayW2', ...
            'Timer', 0,... % time before trial start
            'StateChangeConditions', {'Tup', 'DelayW3'},...
            'OutputActions', {'GlobalTimerTrig', 2});
       sma = AddState(sma, 'Name', 'DelayW3', ...
            'Timer', 2,... % time before trial start
            'StateChangeConditions', {'Tup', 'StimulusDeliver'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'StimulusDeliver', ...
            'Timer', 0,... % Start tone
            'StateChangeConditions', {'Tup', 'CueDelay'},...
            'OutputActions', {'SoftCode', 1});
        sma = AddState(sma, 'Name', 'CueDelay', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'ResponseW1'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'ResponseW1', ...
             'Timer',3,...
            'StateChangeConditions', {'Port1In', 'ResponseW2','Tup', 'ITI' },...
             'OutputActions', {});  
         sma = AddState(sma, 'Name', 'ResponseW2', ...
             'Timer',0,...
            'StateChangeConditions', { 'GlobalTimer2_End', 'Delay'},...
             'OutputActions', {});  
        sma = AddState(sma, 'Name', 'Delay', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'Reward'},...
            'OutputActions', {});    
        sma = AddState(sma, 'Name', 'Reward', ...
            'Timer', 0,... % reward delay
            'StateChangeConditions', {'Tup', 'DeliverReward'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'DeliverReward', ...
            'Timer', RewardValveTime,... % reward amount
            'StateChange Conditions', {'Tup', 'VacuumDelay'},...
            'OutputActions', {'ValveState', 2}); 
        sma = AddState(sma, 'Name', 'VacuumDelay', ...
            'Timer', S.GUI.VacuumDelay,...
            'StateChangeConditions', {'Tup', 'VacuumOn'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'VacuumOn', ...
            'Timer', S.GUI.VacuumOnTime,... % Time for vacuum to be on and remove residual fluids
            'StateChangeConditions', {'Tup', 'ITI'},...
            'OutputActions', {'ValveState', 16}); 
        sma = AddState(sma, 'Name', 'Punishment', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup','ITI'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'ITI', ...
             'Timer', 7+ITI(currentTrial),...
             'StateChangeConditions', {'Tup', 'exit'},...
             'OutputActions', {});

        case 3       %% Training mice to do go-no go %%
        if ImagingFlag(currentTrial)==1 
            sma = SetGlobalTimer(sma,  'TimerID', 1, 'Duration',18, 'OnsetDelay', 2, 'Channel', 'Wire1'); 
             sma = SetGlobalTimer(sma,  'TimerID', 2, 'Duration',3,'OnsetDelay', 2); 
            sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {'GlobalTimerTrig', 1});
        else
            sma = SetGlobalTimer(sma,  'TimerID', 2, 'Duration',3,'OnsetDelay', 2); 
            sma = AddState(sma, 'Name', 'TrialStart',...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {});
        end;
        sma = AddState(sma, 'Name', 'DelayW1', ...
            'Timer', 8,...
            'StateChangeConditions', {'Tup', 'DelayW2'},...
            'OutputActions', {});
       sma = AddState(sma, 'Name', 'DelayW2', ...
            'Timer', 0,... % time before trial start
            'StateChangeConditions', {'Tup', 'DelayW3'},...
            'OutputActions', {'GlobalTimerTrig', 2});
       sma = AddState(sma, 'Name', 'DelayW3', ...
            'Timer', 2,... % time before trial start
            'StateChangeConditions', {'Tup', 'StimulusDeliver'},...
            'OutputActions', {}); 
        sma = AddState(sma, 'Name', 'StimulusDeliver', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'CueDelay'},...
            'OutputActions', {'SoftCode', soundID});
         sma = AddState(sma, 'Name', 'CueDelay', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'ResponseW1'},...
            'OutputActions', {});
         sma = AddState(sma, 'Name', 'ResponseW1', ...
             'Timer',3,...
            'StateChangeConditions', {'Port1In', 'ResponseW2','Tup', 'ITI' },...
             'OutputActions', {});  
         sma = AddState(sma, 'Name', 'ResponseW2', ...
             'Timer',0,...
            'StateChangeConditions', { 'GlobalTimer2_End', 'Delay'},...
             'OutputActions', {});  
         sma = AddState(sma, 'Name', 'Delay', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', LickOutcome},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'Reward', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'DeliverReward'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'Punishment', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'DeliverPunishment'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'DeliverReward', ...
            'Timer', RewardValveTime,...
            'StateChangeConditions', {'Tup', 'VacuumDelay'},...
            'OutputActions', {'ValveState', ValveState});
        sma = AddState(sma, 'Name', 'DeliverPunishment', ...
            'Timer', PunishValveTime,...
            'StateChangeConditions', {'Tup', 'VacuumDelay'},...
            'OutputActions', {'ValveState', ValveState});  
       sma = AddState(sma, 'Name', 'VacuumDelay', ...
            'Timer', S.GUI.VacuumDelay,...
            'StateChangeConditions', {'Tup', 'VacuumOn'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'VacuumOn', ...
            'Timer', S.GUI.VacuumOnTime,... % Time for vacuum to be on and remove residual fluids
            'StateChangeConditions', {'Tup', 'ITI'},...
            'OutputActions', {'ValveState', 16});           
        sma = AddState(sma, 'Name', 'ITI', ...
            'Timer',7+ ITI(currentTrial),...
            'StateChangeConditions', {'Tup', 'exit'},...
            'OutputActions', {});
    
    end
    SendStateMatrix(sma);
    RawEvents = RunStateMatrix;
    
    if ~isempty(fieldnames(RawEvents)) % If trial data was returned
        BpodSystem.Data = AddTrialEvents(BpodSystem.Data,RawEvents); % Computes trial events from raw data
        BpodSystem.Data.TrialSettings(currentTrial) = S; % Adds the settings used for the current trial to the Data struct (to be saved after the trial ends)
        BpodSystem.Data.TrialTypes(currentTrial) = TrialTypes(currentTrial); % Adds the trial type of the current trial to data
        BpodSystem.Data.RewardDelay(currentTrial) = RewardDelay(currentTrial);
        BpodSystem.Data.PunishDelay(currentTrial) = PunishDelay(currentTrial);
        BpodSystem.Data.ITI(currentTrial) = ITI(currentTrial);
        
        %Outcome
        if ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Reward(1))
            BpodSystem.Data.Outcomes(currentTrial) = 1;
        elseif ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Punishment(1))
            BpodSystem.Data.Outcomes(currentTrial) = 0;
        elseif TrialTypes(currentTrial)==1
            BpodSystem.Data.Outcomes(currentTrial) = -1;
        else
            BpodSystem.Data.Outcomes(currentTrial) = 2;
        end

        UpdateTotalRewardDisplay(S.GUI.RewardAmount, currentTrial);
        UpdateGoNoGoOutcomePlot(TrialTypes, BpodSystem.Data);
        SaveBpodSessionData; % Saves the field BpodSystem.Data to the current data file
    end
    HandlePauseCondition; % Checks to see if the protocol is paused. If so, waits until user resumes.
    if BpodSystem.Status.BeingUsed == 0
        return
    end
end

function UpdateGoNoGoOutcomePlot(TrialTypes, Data)
global BpodSystem
Outcomes = zeros(1,Data.nTrials);
for x = 1:Data.nTrials
    if ~isnan(Data.RawEvents.Trial{x}.States.Reward(1))
        Outcomes(x) = 1;
    elseif ~isnan(Data.RawEvents.Trial{x}.States.Punishment(1))
        Outcomes(x) = 0;
    elseif BpodSystem.Data.TrialTypes(x)==1
        Outcomes(x) = -1;
    else
        Outcomes(x) = 2;
    end
end
GoNoGoOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'update',Data.nTrials+1,TrialTypes,Outcomes);

function UpdateTotalRewardDisplay(RewardAmount, currentTrial)
% If rewarded based on the state data, update the TotalRewardDisplay
global BpodSystem
if ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Reward(1))
    TotalRewardDisplay('add', RewardAmount);
end
