function RewardPunishmentCondition_csus_UnexpectReward

global BpodSystem

%% Define parameters
S = BpodSystem.ProtocolSettings; % Load settings chosen in launch manager into current workspace as a struct called S
if isempty(fieldnames(S))  % If settings file was an empty struct, populate struct with default settings
    
    S.GUI.RewardAmount = 10; % ul
    S.GUI.PunishAmount = 1; % s (air puff or shock)
    S.GUI.USType =1;
    S.GUI.ImagingStart = 1;   S.GUI.ImagingNumber = 20;
    
    S.GUI.TrainingLevel = 1; % Configurable training level
    S.ITI = 10;
    S.SoundDuration = 1;
    
    S.CueDelay = 0;
end

LickPort = 'Port3In';
reward_arg = {'ValveState',2};
punish_arg =  {'ValveState', 16};

% Initialize parameter GUI plugin
BpodParameterGUI('init', S);

%% Define trials
MaxTrials = 200;
if S.GUI.USType==1
    TrialTypes = ones(1,MaxTrials);
elseif S.GUI.USType==2
    TrialTypes = 2*ones(1,MaxTrials);
end
ITI = randi(3,1,MaxTrials);

Temp = randi(2,1,20);
Temp1 = ones(1,80);
RewardFlag = [Temp,Temp1]; 

ImagingFlag = zeros(1,MaxTrials);
ImagingFlag(S.GUI.ImagingStart:S.GUI.ImagingNumber +S.GUI.ImagingStart-1) = 1;

BpodSystem.Data.TrialTypes = []; % The trial type of each trial completed will be added here.
BpodSystem.Data.RewardFlag = RewardFlag;
%% Initialize plots
BpodSystem.ProtocolFigures.SideOutcomePlotFig = figure('Position', [100 200 1200 300],'name','Outcome plot','numbertitle','off', 'MenuBar', 'none', 'Resize', 'off');
BpodSystem.GUIHandles.SideOutcomePlot = axes('Position', [.075 .3 .89 .6]);
GoNoGoOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'init',TrialTypes);

% Set soft code handler to trigger sounds
BpodSystem.SoftCodeHandlerFunction = 'SoftCodeHandler_PlaySoundX';

Amp=0.06
SF = 192000; % Sound card sampling rate
SinWaveFreq1 = 4000;
sounddata1 = GenerateSineWave(Amp, SF, SinWaveFreq1, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)
SinWaveFreq2 = 10000;
sounddata2 = GenerateSineWave(Amp, SF, SinWaveFreq2, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)

% Program sound server
PsychToolboxSoundServer('init')
PsychToolboxSoundServer('Load', 1, sounddata1);
PsychToolboxSoundServer('Load', 2, sounddata2);

%% Main trial loop
for currentTrial = 1:MaxTrials
    
    S = BpodParameterGUI('sync', S); % Sync parameters with BpodParameterGUI plugin
    R = GetValveTimes(S.GUI.RewardAmount, [1]);
    RewardValveTime = R; % Update reward amounts
    
    switch TrialTypes(currentTrial) % Determine trial-specific state matrix fields
        case 1
            soundID = 1;
            DOutcome = 'Reward';
        case 2
            soundID = 2;
            DOutcome = 'Punishment';
    end
    
    sma = NewStateMatrix(); % Assemble state matrix
    
  if S.GUI.TrainingLevel==1 % reward or punishment
        if ImagingFlag(currentTrial)==1
            sma = SetGlobalTimer(sma,  'TimerID', 1, 'Duration',20, 'OnsetDelay', 2, 'Channel', 'Wire1');
            
            sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {'GlobalTimerTrig', 1});
        else
            sma = AddState(sma, 'Name', 'TrialStart',...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {});
        end;
        sma = AddState(sma, 'Name', 'DelayW1', ...
            'Timer', 10,...
            'StateChangeConditions', {'Tup', 'StimulusDeliver'},...
            'OutputActions', {});
       if RewardFlag(currentTrial) == 1
       sma = AddState(sma, 'Name', 'StimulusDeliver', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'DelayW2'},...
            'OutputActions', {'SoftCode', soundID});
       elseif RewardFlag(currentTrial) == 2
       sma = AddState(sma, 'Name', 'StimulusDeliver', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'DelayW2'},...
            'OutputActions', {});
       end
       sma = AddState(sma, 'Name', 'DelayW2', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', DOutcome},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'Reward', ...
            'Timer', 2,... 
            'StateChangeConditions', {'Tup', 'DeliverReward'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'DeliverReward',...
            'Timer', RewardValveTime,... 
            'StateChangeConditions', {'Tup', 'ITI'},...
            'OutputActions',  reward_arg); 
        sma = AddState(sma, 'Name', 'Punishment',...
            'Timer', 2,...
            'StateChangeConditions', {'Tup', 'DeliverPunishment'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'DeliverPunishment',...
            'Timer',S.GUI.PunishAmount,...   
            'StateChangeConditions', {'Tup', 'ITI'},...
            'OutputActions', punish_arg); 
        sma = AddState(sma, 'Name', 'ITI',...
            'Timer', 10+ITI(currentTrial),...
            'StateChangeConditions', {'Tup', 'exit'},...
            'OutputActions', {});
        
    end
    
    SendStateMatrix(sma);
    RawEvents = RunStateMatrix;
    
    if ~isempty(fieldnames(RawEvents)) % If trial data was returned
        BpodSystem.Data = AddTrialEvents(BpodSystem.Data,RawEvents); % Computes trial events from raw data
        BpodSystem.Data.TrialSettings(currentTrial) = S; % Adds the settings used for the current trial to the Data struct (to be saved after the trial ends)
        BpodSystem.Data.TrialTypes(currentTrial) = TrialTypes(currentTrial); % Adds the trial type of the current trial to data
        BpodSystem.Data.ITI(currentTrial) = ITI(currentTrial);
        
        %Outcome
        if ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Reward(1))
            BpodSystem.Data.Outcomes(currentTrial) = 1;
        elseif ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Punishment(1))
            BpodSystem.Data.Outcomes(currentTrial) = 0;
        elseif TrialTypes(currentTrial)==1
            BpodSystem.Data.Outcomes(currentTrial) = -1;
        else
            BpodSystem.Data.Outcomes(currentTrial) = 2;
        end
        
        UpdateGoNoGoOutcomePlot(TrialTypes, BpodSystem.Data);
        SaveBpodSessionData; % Saves the field BpodSystem.Data to the current data file
    end
    HandlePauseCondition; % Checks to see if the protocol is paused. If so, waits until user resumes.
    if BpodSystem.Status.BeingUsed == 0
        return
    end
end

function UpdateGoNoGoOutcomePlot(TrialTypes, Data)
global BpodSystem
Outcomes = zeros(1,Data.nTrials);
for x = 1:Data.nTrials
    if ~isnan(Data.RawEvents.Trial{x}.States.Reward(1))
        Outcomes(x) = 1;
    elseif ~isnan(Data.RawEvents.Trial{x}.States.Punishment(1))
        Outcomes(x) = 0;
    elseif BpodSystem.Data.TrialTypes(x)==1
        Outcomes(x) = -1;
    else
        Outcomes(x) = 2;
    end
end
GoNoGoOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'update',Data.nTrials+1,TrialTypes,Outcomes);
