function Reward_2AFC_Imaging

global BpodSystem

%% Define parameters
S = BpodSystem.ProtocolSettings; % Load settings chosen in launch manager into current workspace as a struct called S
if isempty(fieldnames(S))  % If settings file was an empty struct, populate struct with default settings
    
    S.GUI.LeftRewardAmount =5; % ul water
    S.GUI.RightRewardAmount = 5; % ul  sucrose
    S.GUI.USType =1;  % 1: block by block  2 and 3: random trial
    S.GUI.ImagingStart = 1;   S.GUI.ImagingNumber =1;
    
    S.GUI.TrialGoProb = 0.5;  % Precentage of trials are left or right
    S.GUI.TrainingLevelGoProb =0.3;
    

     S.GUI.TrainingLevel =0; % Configurable training level

    S.ITI = 10;
    S.SoundDuration = 1;
    
    S.CueDelay = 0;
end

LeftLickPort = 'Port1In';
RightLickPort = 'Port2In';

% Initialize parameter GUI plugin
BpodParameterGUI('init', S);

%% Define trials
MaxTrials = 100;
if S.GUI.USType==1  % 5 trials left and 5 trials right  block trial
    TrialTypes=[];
    leftAll=ones(1,5);  %ones
    rightAll=ones(1,5); %zeros
    all=repmat([leftAll rightAll],MaxTrials);
    TrialTypes=all(1,:);
    S.GUI.TrainingLevel(1:MaxTrials) = 1;  % 2: Pavlovian conditioning 3: Instrumental learning
    
elseif S.GUI.USType==2 % random trials of left and right
    for ii=1:MaxTrials
        if rand<S.GUI.TrialGoProb
            TrialTypes(ii) = 1;
        else
            TrialTypes(ii) = 0;
        end
    end
  S.GUI.TrainingLevel(1:MaxTrials) = 2;    % 2: Pavlovian conditioning 3: Instrumental learning
  
elseif S.GUI.USType == 3  % random trials of left and right, but some trials Pavlovian and some trials Instrumental
        for ii=1:MaxTrials
        if rand<S.GUI.TrialGoProb
            TrialTypes(ii) = 1;
        else
            TrialTypes(ii) = 0;
        end
      end
       for ii=1:MaxTrials
        if rand< S.GUI.TrainingLevelGoProb
            S.GUI.TrainingLevel(ii) = 2;       % 2: Pavlovian conditioning
        else
            S.GUI.TrainingLevel(ii) = 3;       % 3: Instrumental learning
        end
       end
end

ITI = randi(3,1,MaxTrials);

ImagingFlag = zeros(1,MaxTrials);
ImagingFlag(S.GUI.ImagingStart:S.GUI.ImagingNumber +S.GUI.ImagingStart-1) = 1;

BpodSystem.Data.TrialTypes = []; % The trial type of each trial completed will be added here.

%% Initialize plots
BpodSystem.ProtocolFigures.SideOutcomePlotFig = figure('Position', [100 200 1200 300],'name','Outcome plot','numbertitle','off', 'MenuBar', 'none', 'Resize', 'off');
BpodSystem.GUIHandles.SideOutcomePlot = axes('Position', [.075 .3 .89 .6]);
GoNoGoOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'init',TrialTypes);

% Set soft code handler to trigger sounds
BpodSystem.SoftCodeHandlerFunction = 'SoftCodeHandler_PlaySoundX';

Amp=0.1  
SF = 192000; % Sound card sampling rate
SinWaveFreq1 = 4000;  %4000
sounddata1 = GenerateSineWave(Amp, SF, SinWaveFreq1, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)
SinWaveFreq2 = 10000; %10000
sounddata2 = GenerateSineWave(Amp, SF, SinWaveFreq2, S.SoundDuration); % Sampling freq (hz), Sine frequency (hz), duration (s)

% Program sound server
PsychToolboxSoundServer('init')
PsychToolboxSoundServer('Load', 1, sounddata1);
PsychToolboxSoundServer('Load', 2, sounddata2);

%% Main trial loop
for currentTrial = 1:MaxTrials
    
    S = BpodParameterGUI('sync', S); % Sync parameters with BpodParameterGUI plugin
    
    LeftValveTime = GetValveTimes(S.GUI.LeftRewardAmount, 1);
    RightValveTime = GetValveTimes(S.GUI.RightRewardAmount, 2); % Update reward amounts
    
    switch TrialTypes(currentTrial) % Determine trial-specific state matrix fields
        case 1
            soundID = 1;
            DOutcome = 'LeftReward';
            RewardValveTime = LeftValveTime;
            RewardValveCode = 1;
            
            LeftActionState = 'Reward'; 
            RightActionState = 'IC'; 
        case 0
            soundID = 2;
            DOutcome = 'RightReward';
            RewardValveTime = RightValveTime;
            RewardValveCode = 2;
            
            LeftActionState = 'IC'; 
            RightActionState = 'Reward'; 
    end
    
    sma = NewStateMatrix(); % Assemble state matrix
    
    switch  S.GUI.TrainingLevel(currentTrial)
    case 1    % only water comes from the two spout
           if ImagingFlag(currentTrial)==1
            sma = SetGlobalTimer(sma,  'TimerID', 1, 'Duration',20, 'OnsetDelay', 2, 'Channel', 'Wire1');
            
            sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {'GlobalTimerTrig', 1});
        else
            sma = AddState(sma, 'Name', 'TrialStart',...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {});
        end;
        sma = AddState(sma, 'Name', 'DelayW1', ...
            'Timer', 10,...   
            'StateChangeConditions', {'Tup', 'StimulusDeliver'},...
            'OutputActions', {});
       sma = AddState(sma, 'Name', 'StimulusDeliver', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'CueDelay'},...
            'OutputActions', {});
       sma = AddState(sma, 'Name', 'CueDelay', ...
            'Timer', S.CueDelay,...
            'StateChangeConditions', {'Tup', 'Reward'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'Reward', ...
            'Timer', 2,... 
            'StateChangeConditions', {'Tup', 'DeliverReward'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'DeliverReward',...
            'Timer', RewardValveTime,... 
            'StateChangeConditions', {'Tup', 'ITI'},...
            'OutputActions', {'ValveState', RewardValveCode} ); 
        sma = AddState(sma, 'Name', 'ITI',...
            'Timer', 10+ITI(currentTrial),...
            'StateChangeConditions', {'Tup', 'exit'},...
            'OutputActions', {});
        
     case 2    % tone pairs with each spout
           if ImagingFlag(currentTrial)==1
            sma = SetGlobalTimer(sma,  'TimerID', 1, 'Duration',20, 'OnsetDelay', 2, 'Channel', 'Wire1'); %20
            
            sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {'GlobalTimerTrig', 1});
        else
            sma = AddState(sma, 'Name', 'TrialStart',...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {});
        end;
        sma = AddState(sma, 'Name', 'DelayW1', ...
            'Timer', 10,...
            'StateChangeConditions', {'Tup', 'StimulusDeliver'},...
            'OutputActions', {});
       sma = AddState(sma, 'Name', 'StimulusDeliver', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'CueDelay'},...
            'OutputActions', {'SoftCode', soundID});
       sma = AddState(sma, 'Name', 'CueDelay', ...
            'Timer', S.CueDelay,...
            'StateChangeConditions', {'Tup', 'Reward'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'Reward', ...
            'Timer', 2,... 
            'StateChangeConditions', {'Tup', 'DeliverReward'},...
            'OutputActions', {});
        sma = AddState(sma, 'Name', 'DeliverReward',...
            'Timer', RewardValveTime,... 
            'StateChangeConditions', {'Tup', 'ITI'},...
            'OutputActions', {'ValveState', RewardValveCode} ); 
        sma = AddState(sma, 'Name', 'ITI',...
            'Timer', 10+ITI(currentTrial),...
            'StateChangeConditions', {'Tup', 'exit'},...
            'OutputActions', {});  
        
    case 3   % Mice have to lick spout to get water plus imaging 
        if ImagingFlag(currentTrial)==1
            sma = SetGlobalTimer(sma,  'TimerID', 1, 'Duration',14, 'OnsetDelay', 2, 'Channel', 'Wire1'); %20
             sma = SetGlobalTimer(sma,  'TimerID', 2, 'Duration',3,'OnsetDelay', 2); 
            sma = AddState(sma, 'Name', 'TrialStart', ...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {'GlobalTimerTrig', 1});
        else
            sma = SetGlobalTimer(sma,  'TimerID', 2, 'Duration',3,'OnsetDelay', 2); 
            sma = AddState(sma, 'Name', 'TrialStart',...
                'Timer', 0,...
                'StateChangeConditions', {'Tup', 'DelayW1'},...
                'OutputActions', {});
        end;
         sma = AddState(sma, 'Name', 'DelayW1', ...
            'Timer', 6,...
            'StateChangeConditions', {'Tup', 'Delay'},...
            'OutputActions', {});     
        sma = AddState(sma, 'Name', 'Delay', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'DelayW3'},...
            'OutputActions', {'GlobalTimerTrig', 2});
         sma = AddState(sma, 'Name', 'DelayW3', ...
            'Timer', 2,... 
            'StateChangeConditions', {'Tup', 'StimulusDeliver'},...
            'OutputActions', {}); 
       sma = AddState(sma, 'Name', 'StimulusDeliver', ...
            'Timer', 0,...
            'StateChangeConditions', {'Tup', 'Hold'},...
            'OutputActions', {'SoftCode', soundID});
       sma = AddState(sma, 'Name', 'Hold', ...
        'Timer', 0,...***
        'StateChangeConditions', {'Tup', 'GoSignal'},...
        'OutputActions', {}); 
    %Change between Free water or choice.
    sma = AddState(sma, 'Name', 'GoSignal', ...
        'Timer', 0,...
        'StateChangeConditions', {'Tup', 'Choice'},...    
        'OutputActions', {});  
       sma = AddState(sma, 'Name', 'Choice', ...   
        'Timer', 3,...
        'StateChangeConditions', {'Tup', 'ITI',...
        'Port1In', LeftActionState,...
        'Port2In', RightActionState},...
        'OutputActions', {}); 
        sma = AddState(sma, 'Name', 'Reward',...
            'Timer', 0,... 
            'StateChangeConditions', { 'GlobalTimer2_End', 'DeliverReward'},...
            'OutputActions', {}); 
	  sma = AddState(sma, 'Name', 'DeliverReward',...
	   'Timer', RewardValveTime,... 
	   'StateChangeConditions', {'Tup', 'ITI'},...
	   'OutputActions', {'ValveState', RewardValveCode} ); 
       sma = AddState(sma, 'Name', 'IC', ...
        'Timer', 0,...
        'StateChangeConditions', {'GlobalTimer2_End', 'ITI'},...   
        'OutputActions', {}); 
      sma = AddState(sma, 'Name', 'NoResponse', ...
        'Timer', 0,...  
        'StateChangeConditions', {'Tup', 'ITI'},...
        'OutputActions', {}); 
       sma = AddState(sma, 'Name', 'ITI',...
            'Timer', 5+ITI(currentTrial),...
            'StateChangeConditions', {'Tup', 'exit'},...
            'OutputActions', {});
        
    end

    SendStateMatrix(sma);
    RawEvents = RunStateMatrix;
    
    if ~isempty(fieldnames(RawEvents)) % If trial data was returned
        BpodSystem.Data = AddTrialEvents(BpodSystem.Data,RawEvents); % Computes trial events from raw data
        BpodSystem.Data.TrialSettings(currentTrial) = S; % Adds the settings used for the current trial to the Data struct (to be saved after the trial ends)
        BpodSystem.Data.TrialTypes(currentTrial) = TrialTypes(currentTrial); % Adds the trial type of the current trial to data
        BpodSystem.Data.ITI(currentTrial) = ITI(currentTrial);
        
        %Outcome
        if ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.Reward(1))
            BpodSystem.Data.Outcomes(currentTrial) = 1;
        elseif ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.IC(1))    
            BpodSystem.Data.Outcomes(currentTrial) = 2;
        elseif ~isnan(BpodSystem.Data.RawEvents.Trial{currentTrial}.States.NoResponse(1))    
           BpodSystem.Data.Outcomes(currentTrial) = -1;
        end
        
        UpdateGoNoGoOutcomePlot(TrialTypes, BpodSystem.Data);
        SaveBpodSessionData; % Saves the field BpodSystem.Data to the current data file
    end
    HandlePauseCondition; % Checks to see if the protocol is paused. If so, waits until user resumes.
    if BpodSystem.Status.BeingUsed == 0
        return
    end
end

function UpdateGoNoGoOutcomePlot(TrialTypes, Data)
global BpodSystem
Outcomes = zeros(1,Data.nTrials);
for x = 1:Data.nTrials
    if ~isnan(Data.RawEvents.Trial{x}.States.Reward(1))
        Outcomes(x) = 1;
    elseif ~isnan(Data.RawEvents.Trial{x}.States.IC(1))
        Outcomes(x) = 2;
    elseif ~isnan(Data.RawEvents.Trial{x}.States.NoResponse(1))
        Outcomes(x) = -1;
%     else
%         Outcomes(x) = -1;
    end
end
GoNoGoOutcomePlot(BpodSystem.GUIHandles.SideOutcomePlot,'update',Data.nTrials+1,TrialTypes,Outcomes);
