%% PCA and clustering analysis
clear all

%% Run PCA
data = [A B C];  % A, B, C are z scored single neuron acitivty datasets in response to different stimuli
[eigvect,proj,eigval] = pca(data);
proj123 = proj(:,1:3); % choose the number of components which capture explained variance

%% Hierarchical clustering
nCluster = 3;  % choose cluster number which depends on the within-clustering sum of distance 
figure;
cosD = pdist(proj123,'correlation');
clustTreeCos = linkage(cosD,'complete');
cophenet(clustTreeCos,cosD)
[h,nodes] = dendrogram(clustTreeCos,0);
h_gca = gca;
h_gca.TickDir = 'out';
h_gca.TickLength = [.002 0];
T = cluster(clustTreeCos,'maxclust',nCluster);
dendrogram(clustTreeCos,0.1)

%% sort and plot heatmap figure
n1= -50; n2=50;   
[A B] = sort(T,'descend');
ranking = B;
for i = 1:length(ranking)
   data_ranking(i,:) = data(ranking(i),:);
   proj123_ranking(i,:) = proj123(ranking(i),:);
   Zscore1_ranking(i,:) = A(ranking(i),:); 
   Zscore2_ranking(i,:) = B(ranking(i),:);
   Zscore3_ranking(i,:) = C(ranking(i),:);
end

% Plot heatmap of ranked auROC
% colorspace
Parameter.colorR = 28; % for colorspace, do not need to change
Parameter.colorC = 28; % for colorspace, do not need to change
colorspace = zeros(64,3);
colorspace(64-Parameter.colorR+1:64,1) = linspace(0,1,Parameter.colorR); % RGB color space, define R value here
colorspace(1:Parameter.colorC,2) = linspace(1,0,Parameter.colorC); % G
colorspace(1:Parameter.colorC,3) = linspace(1,0,Parameter.colorC); % B

figure,imagesc(data_ranking,[n1 n2]);
colormap(colorspace)
% 
figure,imagesc(Zscore1_ranking,[n1 n2]);
colormap(colorspace)
figure,imagesc(Zscore2_ranking,[n1 n2]);
colormap(colorspace)
figure,imagesc(Zscore3_ranking,[n1 n2]);
colormap(colorspace)
%% plot PCA result
enlagre_factor = ones(10,200);
K = kron(proj123_ranking,enlagre_factor);
h3 = figure; imshow(K,[min(K(:))*0.5 max(K(:))*0.5])
set(gcf,'color','[1 1 1]')