%% Decoding by SVM and plot data %%
%Train the SVM Classifier
dataset = proj_decoding;

n_tri1 = 20;       
n_tri2 = 20;
trial_type = [ones(n_tri1,1);-1*ones(n_tri2,1)];

cl =  fitcsvm(dataset,trial_type,'KernelFunction','linear','Standardize',true,'KernelScale','auto');
% Predict scores over the grid
d = 0.02;
[x1Grid,x2Grid] = meshgrid(min(dataset(:,1)):d:max(dataset(:,1)),...
    min(dataset(:,2)):d:max(dataset(:,2)));
xGrid = [x1Grid(:),x2Grid(:)];
[~,scores] = predict(cl,xGrid);

% Plot the data and the decision boundary
figure;
h(1:2) = gscatter(dataset(:,1),dataset(:,2),trial_type,'rb','.');
hold on
contour(x1Grid,x2Grid,reshape(scores(:,2),size(x1Grid)),[0 0],'k');
legend(h,{'-1','+1','Support Vectors'});
axis equal
hold off

%% Decoding by SVM and predict %%
% calculate the accuracy
dataset = proj_decoding;

n_tri1 =20;
n_tri2 =20;
trial_type = [ones(n_tri1,1);-1*ones(n_tri2,1)];

n_triT1 = fix(0.75*n_tri1);  % select 75% of trials to train the classifier
n_triT2 = fix(0.75*n_tri2);

n_triTest1 = n_tri1-n_triT1; % The remaining 25% of trials are used to test decoding accuracy
n_triTest2 = n_tri2-n_triT2;
RepeatN = 1000;

ACC = [];
ACC_permutation = [];

for i=size(dataset,2)
    
%     data_temp = dataset(:,i,:);
    data = dataset;
    
    D1 = data(1:n_tri1,:);
    D2 = data((n_tri1+1):(n_tri1+n_tri2),:);
    
    AccurateFlag = [];
    AccurateFlag_permutation = [];

    for ii=1:1000
       
        % random choose 75% trials as train data and 25% trials as test
        % data, and train the classifer with train data and calcualte the
        % decoding accurace with test data, iteration of 1000 times.   
        sel1 = randperm(n_tri1,n_triT1);            
        sel2 = randperm(n_tri2,n_triT2);
        data_train = [D1(sel1,:);D2(sel2,:)];
        theclass_train = [ones(length(sel1),1);-1*ones(length(sel2),1)];
        
        sel1B = setdiff(1:n_tri1,sel1);
        sel2B = setdiff(1:n_tri2,sel2);
        sel1B = randsample(sel1B,n_triTest1);
        sel2B = randsample(sel2B,n_triTest2);
        data_test = [D1(sel1B,:);D2(sel2B,:)];
        theclass_test = [ones(length(sel1B),1);-1*ones(length(sel2B),1)];
        
        cl = fitcsvm(data_train,theclass_train,'KernelFunction','linear','Standardize',true,'KernelScale','auto');
        [label,~] = predict(cl,data_test);
        
        AccurateFlag0 = (theclass_test==label);
        AccurateFlag = [AccurateFlag;AccurateFlag0];
        
        % permutation of trial A and trial B sequence, calculate the same
        % decocding accuracy, iteration of 1000 times. 
        D_permuation = [D1;D2];
        sel_p = randperm(n_tri1+n_tri2,n_tri1+n_tri2);
        D_permuation = D_permuation(sel_p,:);
        D1_perm = D_permuation(1:n_tri1,:);
        D2_perm = D_permuation(n_tri1+1:end,:);
        
        sel1 = randperm(n_tri1,n_triT1);
        sel2 = randperm(n_tri2,n_triT2);
        data_train = [D1_perm(sel1,:);D2_perm(sel2,:)];
        theclass_train = [ones(length(sel1),1);-1*ones(length(sel2),1)];
        
        sel1B = setdiff(1:n_tri1,sel1);
        sel2B = setdiff(1:n_tri2,sel2);
        sel1B = randsample(sel1B,n_triTest1);
        sel2B = randsample(sel2B,n_triTest2);
        data_test = [D1_perm(sel1B,:);D2_perm(sel2B,:)];
        theclass_test = [ones(length(sel1B),1);-1*ones(length(sel2B),1)];
        
        cl = fitcsvm(data_train,theclass_train,'KernelFunction','linear','Standardize',true,'KernelScale','auto');
        [label,~] = predict(cl,data_test);
        
        AccurateFlag0 = (theclass_test==label);
        AccurateFlag_permutation = [AccurateFlag_permutation;AccurateFlag0];
        
    end
    
    ACC(i) = mean(AccurateFlag);   % accuracy of normal sequence 
   ACC_permutation(i) = mean(AccurateFlag_permutation); % accuracy of permutation 

end

AccSum = ACC;
AccSum_permutation = ACC_permutation;